==> neural_data_simulator-0.2.7-py3-none-any.whl: decoder/run_decoder.py <==
"""Script that configures and starts the example :class:`Decoder`."""

import argparse
import logging
from pathlib import Path
from typing import cast

from decoder.decoders import Decoder
from decoder.decoders import PersistedFileDecoderModel
from decoder.settings import DecoderSettings
from pydantic_yaml import VersionedYamlModel

from neural_data_simulator import inputs
from neural_data_simulator import outputs
from neural_data_simulator import timing
from neural_data_simulator.outputs import StreamConfig
from neural_data_simulator.settings import LogLevel
from neural_data_simulator.settings import TimerModel
from neural_data_simulator.util.runtime import configure_logger
from neural_data_simulator.util.runtime import get_abs_path
from neural_data_simulator.util.runtime import initialize_logger
from neural_data_simulator.util.runtime import open_connection
from neural_data_simulator.util.settings_loader import get_script_settings

SCRIPT_NAME = "nds-decoder"
logger = logging.getLogger(__name__)


class _Settings(VersionedYamlModel):
    """Decoder app settings."""

    log_level: LogLevel
    decoder: DecoderSettings
    timer: TimerModel


def _setup_LSL_input(stream_name: str, connection_timeout: float) -> inputs.LSLInput:
    """Prepare LSL input to read the raw data stream.

    Args:
        stream_name: LSL stream name.
        connection_timeout: Maximum time for attempting
        a connection to the LSL input stream.

    Returns:
        The LSL stream input that can be used to read data from.
    """
    data_input = inputs.LSLInput(stream_name, connection_timeout)
    return data_input


def _setup_LSL_output(
    output_settings: DecoderSettings.Output,
) -> outputs.LSLOutputDevice:
    """Prepare output that will make the data available via an LSL stream.

    Args:
        output_settings: Decoder output settings.

    Returns:
        An LSL output stream that can be used by the decoder to publish data.
    """
    lsl_settings = output_settings.lsl
    stream_config = StreamConfig.from_lsl_settings(
        lsl_settings, output_settings.sampling_rate, output_settings.n_channels
    )
    lsl_output = outputs.LSLOutputDevice(stream_config)
    return lsl_output


def _setup_decoder(
    model_file_path: str,
    input_sample_rate: float,
    output_sample_rate: float,
    n_channels: int,
    spike_threshold: float,
) -> Decoder:
    """Initialize the decoder.

    Args:
        model_file_path: Path to the model file.
        input_sample_rate: Input sample rate.
        output_sample_rate: Output sample rate.
        n_channels: Number of channels.
        spike_threshold: Spike threshold.
    """
    model = PersistedFileDecoderModel(model_file_path)
    decoder = Decoder(
        model, input_sample_rate, output_sample_rate, n_channels, spike_threshold
    )

    return decoder


def _parse_args_settings_path() -> Path:
    """Parse command-line arguments for the settings path."""
    parser = argparse.ArgumentParser(description="Run decoder.")
    parser.add_argument(
        "--settings-path",
        type=Path,
        help="Path to the settings_decoder.yaml file.",
    )
    args = parser.parse_args()
    return args.settings_path


def run():
    """Run the decoder loop."""
    initialize_logger(SCRIPT_NAME)

    settings = cast(
        _Settings,
        get_script_settings(
            _parse_args_settings_path(), "settings_decoder.yaml", _Settings
        ),
    )
    configure_logger(SCRIPT_NAME, settings.log_level)

    timer_settings = settings.timer
    timer = timing.get_timer(timer_settings.loop_time, timer_settings.max_cpu_buffer)

    data_output = _setup_LSL_output(settings.decoder.output)
    lsl_input_settings = settings.decoder.input.lsl
    data_input = _setup_LSL_input(
        lsl_input_settings.stream_name, lsl_input_settings.connection_timeout
    )
    input_sample_rate = data_input.get_info().sample_rate
    n_channels = data_input.get_info().channel_count
    output_sample_rate = 1.0 / timer_settings.loop_time

    dec = _setup_decoder(
        get_abs_path(settings.decoder.model_file),
        input_sample_rate,
        output_sample_rate,
        n_channels,
        settings.decoder.spike_threshold,
    )

    try:
        with open_connection(data_output), open_connection(data_input):
            timer.start()
            while True:
                samples = data_input.read()
                if not samples.empty:
                    samples = dec.decode(samples)
                    if not samples.empty:
                        data_output.send(samples)
                timer.wait()
    except KeyboardInterrupt:
        logger.info("CTRL+C received. Exiting...")


if __name__ == "__main__":
    run()

==> neural_data_simulator-0.2.8-py3-none-any.whl: decoder/run_decoder.py <==
"""Script that configures and starts the example :class:`Decoder`."""

import argparse
import logging
from pathlib import Path
from typing import cast

from decoder.decoders import Decoder
from decoder.decoders import PersistedFileDecoderModel
from decoder.settings import DecoderSettings
from pydantic_yaml import VersionedYamlModel

from neural_data_simulator import inputs
from neural_data_simulator import outputs
from neural_data_simulator import timing
from neural_data_simulator.settings import LogLevel
from neural_data_simulator.settings import TimerModel
from neural_data_simulator.util.runtime import configure_logger
from neural_data_simulator.util.runtime import get_abs_path
from neural_data_simulator.util.runtime import initialize_logger
from neural_data_simulator.util.runtime import open_connection
from neural_data_simulator.util.settings_loader import get_script_settings

SCRIPT_NAME = "nds-decoder"
logger = logging.getLogger(__name__)


class _Settings(VersionedYamlModel):
    """Decoder app settings."""

    log_level: LogLevel
    decoder: DecoderSettings
    timer: TimerModel


def _parse_args_settings_path() -> Path:
    """Parse command-line arguments for the settings path."""
    parser = argparse.ArgumentParser(description="Run decoder.")
    parser.add_argument(
        "--settings-path",
        type=Path,
        help="Path to the settings_decoder.yaml file.",
    )
    args = parser.parse_args()
    return args.settings_path


def _read_decode_send(
    data_input: inputs.Input, dec: Decoder, data_output: outputs.Output
) -> None:
    """Read input data, decode it, and send to the output stream.

    Args:
        data_input: Input data source.
        dec: Decoder.
        data_output: Output data sink.
    """
    samples = data_input.read()
    if not samples.empty:
        decoded_samples = dec.decode(samples)
        if not decoded_samples.empty:
            data_output.send(decoded_samples)


def run():
    """Run the decoder loop."""
    initialize_logger(SCRIPT_NAME)

    settings = cast(
        _Settings,
        get_script_settings(
            _parse_args_settings_path(), "settings_decoder.yaml", _Settings
        ),
    )
    configure_logger(SCRIPT_NAME, settings.log_level)

    # Set up timer
    timer_settings = settings.timer
    timer = timing.get_timer(timer_settings.loop_time, timer_settings.max_cpu_buffer)

    # Create LSL input and output objects
    output_settings = settings.decoder.output
    data_output = outputs.LSLOutputDevice.from_lsl_settings(
        lsl_settings=output_settings.lsl,
        sampling_rate=output_settings.sampling_rate,
        n_channels=output_settings.n_channels,
    )
    lsl_input_settings = settings.decoder.input.lsl
    data_input = inputs.LSLInput(
        stream_name=lsl_input_settings.stream_name,
        connection_timeout=lsl_input_settings.connection_timeout,
    )

    # Set up decoder
    decoder_model = PersistedFileDecoderModel(get_abs_path(settings.decoder.model_file))
    dec = Decoder(
        model=decoder_model,
        input_sample_rate=data_input.get_info().sample_rate,
        output_sample_rate=1.0 / timer_settings.loop_time,
        n_channels=data_input.get_info().channel_count,
        threshold=settings.decoder.spike_threshold,
    )

    try:
        with open_connection(data_output), open_connection(data_input):
            timer.start()
            # Run the decoder periodically
            while True:
                _read_decode_send(
                    data_input=data_input, dec=dec, data_output=data_output
                )
                timer.wait()
    except KeyboardInterrupt:
        logger.info("CTRL+C received. Exiting...")


if __name__ == "__main__":
    run()
